@@ -1,4 +1,3 @@
-#!/usr/bin/env python
 # Copyright 2012-2023, Andrey Kislyuk and argcomplete contributors. Licensed under the terms of the
 # `Apache License, Version 2.0 <http://www.apache.org/licenses/LICENSE-2.0>`_. Distribution of the LICENSE and NOTICE
 # files with source copies of this package and derivative works is **REQUIRED** as specified by the Apache License.
@@ -6,7 +5,7 @@
 
 from shlex import quote
 
-bashcode = r"""
+bashcode = r"""#compdef %(executables)s
 # Run something, muting output or redirecting it to the debug stream
 # depending on the value of _ARC_DEBUG.
 # If ARGCOMPLETE_USE_TEMPFILES is set, use tempfiles for IPC.
@@ -25,15 +24,22 @@ __python_argcomplete_run() {
 
 __python_argcomplete_run_inner() {
     if [[ -z "${_ARC_DEBUG-}" ]]; then
-        "$@" 8>&1 9>&2 1>/dev/null 2>&1
+        "$@" 8>&1 9>&2 1>/dev/null 2>&1 </dev/null
     else
-        "$@" 8>&1 9>&2 1>&9 2>&1
+        "$@" 8>&1 9>&2 1>&9 2>&1 </dev/null
     fi
 }
 
 _python_argcomplete%(function_suffix)s() {
     local IFS=$'\013'
+    local script="%(argcomplete_script)s"
     if [[ -n "${ZSH_VERSION-}" ]]; then
+        if [[ "${_matcher_num-}" -gt 1 ]]; then
+            # Return early if the completer is called multiple times in the same completion run.
+            # Currently the only known occurrence of this is in zsh when a matcher-list zstyle is declared.
+            # When this happens, _matcher_num is incremented past 1.
+            return
+        fi
         local completions
         completions=($(IFS="$IFS" \
             COMP_LINE="$BUFFER" \
@@ -41,8 +47,16 @@ _python_argcomplete%(function_suffix)s() {
             _ARGCOMPLETE=1 \
             _ARGCOMPLETE_SHELL="zsh" \
             _ARGCOMPLETE_SUPPRESS_SPACE=1 \
-            __python_argcomplete_run "${words[1]}") )
-        _describe "${words[1]}" completions -o nosort
+            __python_argcomplete_run ${script:-${words[1]}}))
+        local nosort=()
+        local nospace=()
+        if is-at-least 5.8; then
+            nosort=(-o nosort)
+        fi
+        if [[ "${completions-}" =~ ([^\\]): && "${match[1]}" =~ [=/:] ]]; then
+            nospace=(-S '')
+        fi
+        _describe "${words[1]}" completions "${nosort[@]}" "${nospace[@]}"
     else
         local SUPPRESS_SPACE=0
         if compopt +o nospace 2> /dev/null; then
@@ -56,7 +70,7 @@ _python_argcomplete%(function_suffix)s() {
             _ARGCOMPLETE=1 \
             _ARGCOMPLETE_SHELL="bash" \
             _ARGCOMPLETE_SUPPRESS_SPACE=$SUPPRESS_SPACE \
-            __python_argcomplete_run "%(argcomplete_script)s"))
+            __python_argcomplete_run ${script:-$1}))
         if [[ $? != 0 ]]; then
             unset COMPREPLY
         elif [[ $SUPPRESS_SPACE == 1 ]] && [[ "${COMPREPLY-}" =~ [=/:]$ ]]; then
@@ -67,7 +81,16 @@ _python_argcomplete%(function_suffix)s() {
 if [[ -z "${ZSH_VERSION-}" ]]; then
     complete %(complete_opts)s -F _python_argcomplete%(function_suffix)s %(executables)s
 else
-    compdef _python_argcomplete%(function_suffix)s %(executables)s
+    # When called by the Zsh completion system, this will end with
+    # "loadautofunc" when initially autoloaded and "shfunc" later on, otherwise,
+    # the script was "eval"-ed so use "compdef" to register it with the
+    # completion system
+    autoload is-at-least
+    if [[ $zsh_eval_context == *func ]]; then
+        _python_argcomplete%(function_suffix)s "$@"
+    else
+        compdef _python_argcomplete%(function_suffix)s %(executables)s
+    fi
 fi
 """
 
@@ -105,13 +128,13 @@ Register-ArgumentCompleter -Native -CommandName %(executable)s -ScriptBlock {
     $env:_ARGCOMPLETE = 1
     $env:_ARGCOMPLETE_SUPPRESS_SPACE = 0
     $env:_ARGCOMPLETE_IFS = "`n"
+    $env:_ARGCOMPLETE_SHELL = "powershell"
     %(argcomplete_script)s 2>&1 | Out-Null
 
     Get-Content $completion_file | ForEach-Object {
         [System.Management.Automation.CompletionResult]::new($_, $_, "ParameterValue", $_)
     }
-    rm $completion_file
-    Remove-Item Env:\_ARGCOMPLETE_STDOUT_FILENAME, Env:\ARGCOMPLETE_USE_TEMPFILES, Env:\COMP_LINE, Env:\COMP_POINT, Env:\_ARGCOMPLETE, Env:\_ARGCOMPLETE_SUPPRESS_SPACE, Env:\_ARGCOMPLETE_IFS
+    Remove-Item $completion_file, Env:\_ARGCOMPLETE_STDOUT_FILENAME, Env:\ARGCOMPLETE_USE_TEMPFILES, Env:\COMP_LINE, Env:\COMP_POINT, Env:\_ARGCOMPLETE, Env:\_ARGCOMPLETE_SUPPRESS_SPACE, Env:\_ARGCOMPLETE_IFS, Env:\_ARGCOMPLETE_SHELL
 }
 """  # noqa: E501
 
@@ -123,9 +146,10 @@ def shellcode(executables, use_defaults=True, shell="bash", complete_arguments=N
     Provide the shell code required to register a python executable for use with the argcomplete module.
 
     :param list(str) executables: Executables to be completed (when invoked exactly with this name)
-    :param bool use_defaults: Whether to fallback to readline's default completion when no matches are generated.
+    :param bool use_defaults: Whether to fallback to readline's default completion when no matches are generated
+        (affects bash only)
     :param str shell: Name of the shell to output code for
-    :param complete_arguments: Arguments to call complete with
+    :param complete_arguments: Arguments to call complete with (affects bash only)
     :type complete_arguments: list(str) or None
     :param argcomplete_script: Script to call complete with, if not the executable to complete.
         If supplied, will be used to complete *all* passed executables.
@@ -142,9 +166,10 @@ def shellcode(executables, use_defaults=True, shell="bash", complete_arguments=N
         executables_list = " ".join(quoted_executables)
         script = argcomplete_script
         if script:
-            function_suffix = "_" + script
+            # If the script path contain a space, this would generate an invalid function name.
+            function_suffix = "_" + script.replace(" ", "_SPACE_")
         else:
-            script = "$1"
+            script = ""
             function_suffix = ""
         code = bashcode % dict(
             complete_opts=complete_options,
